TITAN_AFKTIMER_ID = "AFKTimer"

local L = LibStub("AceLocale-3.0"):GetLocale("Titan", true)
local AceTimer = LibStub("AceTimer-3.0")
local AFKTimerTimer, AFKNote


function TitanPanelAFKTimerButton_OnLoad(self)
	self.registry = {
		id = TITAN_AFKTIMER_ID,
		version = GetAddOnMetadata("TitanAFKTimer", "Version"),
		menuText = "AFK Timer",
		category = "Interface",
		buttonTextFunction = "TitanPanelAFKTimerButton_GetButtonText",
		tooltipTitle = "AFK Timer Info",
		tooltipTextFunction = "TitanPanelAFKTimerButton_GetTooltipText",
		icon = "Interface\\Icons\\Spell_Nature_TimeStop",
		iconWidth = 16,
		savedVariables = {
			ShowDuration = 1,
			LocalTime = false,
			ShowIcon = 1,
		}
	}
	
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("CHAT_MSG_SYSTEM")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
end


function TitanPanelAFKTimerButton_OnEvent(self, event, ...)
	if event == "PLAYER_LEAVING_WORLD" then
		-- when "leaving world" AFK is always cancelled, but no system message is sent, so we need a custom made check
		if not UnitIsAFK("player") and TitanAFKTimerData then
			TitanPanelAFKTimer_EndAFK()
			DEFAULT_CHAT_FRAME:AddMessage(CLEARED_AFK, 1, 1, 0)
			AFKNote = nil
			TitanPanelButton_UpdateButton(TITAN_AFKTIMER_ID)
			TitanPanelButton_UpdateTooltip(self)
		end
	elseif event == "ADDON_LOADED" then
		if (...) == "TitanAFKTimer" then
			-- use saved variables to remember AFK session after a UI reload
			if TitanAFKTimerData then
				AFKNote = TitanAFKTimerData.note
				AFKTimerTimer = AceTimer.ScheduleRepeatingTimer("TitanPanel"..TITAN_AFKTIMER_ID, TitanPanelPluginHandle_OnUpdate, 1, {TITAN_AFKTIMER_ID, TITAN_PANEL_UPDATE_BUTTON})
			end
		end
	else
		-- try to extract an AFK message from the system message...
		AFKNote = (...):match(gsub(MARKED_AFK_MESSAGE, "%%s", "(.+)"))
		
		-- ...and if it worked, it was an indication of the player going AFK
		if AFKNote then
			-- Create and start timer
			if TitanPanelAFKTimerButton:IsVisible() and not AFKTimerTimer then
				AFKTimerTimer = AceTimer.ScheduleRepeatingTimer("TitanPanel"..TITAN_AFKTIMER_ID, TitanPanelPluginHandle_OnUpdate, 1, {TITAN_AFKTIMER_ID, TITAN_PANEL_UPDATE_BUTTON})
			end
			
			-- get time stamp when AFK starts
			local AFKStartHr, AFKStartMin
			
			-- local or server time?
			if TitanGetVar(TITAN_AFKTIMER_ID, "LocalTime") then
				AFKStartHr, AFKStartMin = date("*t").hour, date("*t").min
			else
				AFKStartHr, AFKStartMin = GetGameTime()
			end
			
			-- create and initiate variables
			TitanAFKTimerData = {
				startTimeSystem = GetTime(), -- get system uptime when AFK starts
				startTimeStamp = format("%d:%.2d", AFKStartHr, AFKStartMin),
				note = AFKNote,
			}
		elseif (...) == CLEARED_AFK then
			TitanPanelAFKTimer_EndAFK()
		end
		
		TitanPanelButton_UpdateButton(TITAN_AFKTIMER_ID)
		TitanPanelButton_UpdateTooltip(self)
	end
end


-- sets button text
function TitanPanelAFKTimerButton_GetButtonText(id)
	local buttonText
	if TitanAFKTimerData then
		buttonText = "AFK since "..TitanAFKTimerData.startTimeStamp..(TitanGetVar(TITAN_AFKTIMER_ID, "ShowDuration") and " ("..TitanPanelAFKTimer_ParseSeconds(GetTime() - TitanAFKTimerData.startTimeSystem)..")" or "")
	else
		buttonText = "Not AFK"
	end
	return TitanUtils_GetHighlightText(buttonText)
end


-- set tooltip text
function TitanPanelAFKTimerButton_GetTooltipText()
	local tooltipText
	if AFKNote then
		tooltipText = "You are AFK: "..TitanUtils_GetHighlightText(AFKNote)
	else
		tooltipText = "You are not AFK."
	end
	return " \n"..tooltipText.."\n \n"..TitanUtils_GetGreenText("Hint: Left-click to toggle AFK status.")
end


function TitanPanelAFKTimer_EndAFK()
	-- destroy timer
	AceTimer.CancelTimer("TitanPanel"..TITAN_AFKTIMER_ID, AFKTimerTimer, true)
	AFKTimerTimer = nil
	
	-- print AFK time
	DEFAULT_CHAT_FRAME:AddMessage("You were AFK for "..TitanPanelAFKTimer_ParseSeconds(GetTime() - TitanAFKTimerData.startTimeSystem, true).." (since "..TitanAFKTimerData.startTimeStamp..")", 1, 1, 0)
	
	-- destroy variables
	TitanAFKTimerData = nil
end


-- converts seconds into minutes and seconds
function TitanPanelAFKTimer_ParseSeconds(duration, verbose)
	local minutes, seconds
	if duration >= 60 then
		minutes = floor(duration / 60)
		seconds = duration % 60
	else
		minutes = 0
		seconds = duration
	end
	
	seconds = floor(seconds + 0.5)
	
	if verbose then
		return (minutes > 0 and minutes.." |4minute:minutes; and " or "")..seconds.." |4second:seconds;."
	else
		return format("%d:%.2d", minutes, seconds)
	end
end


-- assemble right-click menu
function TitanPanelRightClickMenu_PrepareAFKTimerMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_AFKTIMER_ID].menuText)
	
	TitanPanelRightClickMenu_AddToggleVar("Show duration", TITAN_AFKTIMER_ID, "ShowDuration")
	TitanPanelRightClickMenu_AddToggleVar("Use local time", TITAN_AFKTIMER_ID, "LocalTime")

	TitanPanelRightClickMenu_AddSpacer()
	
	TitanPanelRightClickMenu_AddToggleIcon(TITAN_AFKTIMER_ID)
	TitanPanelRightClickMenu_AddCommand(L["TITAN_PANEL_MENU_HIDE"], TITAN_AFKTIMER_ID, TITAN_PANEL_MENU_FUNC_HIDE)
end


-- toggle AFK status on click
function TitanPanelAFKTimerButton_OnClick(self, button)
	if button == "LeftButton" then
		SendChatMessage("", "AFK")
	end
end